#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "lights.h"

#define ZERO_ADC    1

//#define NUM_ADCS	ZERO_ADC
#define NUM_ADCS	1

volatile static unsigned char current_adc, current_slow_adc;
static uint16_t adc_sum, read_zero, drop_count, read_count, n_reads_log;

static void setup_mux(unsigned char n)
{
	/* ADC numbering: PWM LEDs first, then others, zero at the end */
	switch (n) {
	case 0: // pwmled 1: 1.1V, ADC3 (PB3), single-ended
		ADMUX = _BV(REFS1) | _BV(MUX1) | _BV(MUX0);
		break;
	case ZERO_ADC: // zero: 1.1V, GND, single-ended
		ADMUX = _BV(REFS1) | _BV(MUX3) | _BV(MUX2) | _BV(MUX0);
		break;
	}
}

void start_next_adc()
{
#if 0
	if (current_adc == 0) {
		if (current_slow_adc > N_PWMLEDS) {
			// read one of the non-PWMLED ADCs
			current_adc = --current_slow_adc;
		} else {
			// no more non-PWMLEDs to do, start with PWMLEDs
			current_adc = N_PWMLEDS-1;
		}
	} else if (current_adc >= N_PWMLEDS) {
		// one of the non-PWMLED ADCs just finished, skip to PWMLEDs.
		current_adc = N_PWMLEDS-1;
	} else {
		// next PWMLED
		current_adc--;
	}
#else
	// single ADC for testing only
	current_adc = 0;
#endif

#if 0
	log_byte(0x90 + current_adc); // debug ADC switching
#endif

	adc_sum = 0;
	read_zero = 0;
	drop_count = 1;

	read_count = 1 << PWMLED_ADC_SHIFT;
	n_reads_log = PWMLED_ADC_SHIFT;

	// set up mux, start one-shot conversion
	if (read_zero)
		setup_mux(ZERO_ADC);
	else
		setup_mux(current_adc);

	ADCSRA |= _BV(ADSC);
}

#if 0
void timer_start_slow_adcs()
{
	if (current_slow_adc > N_PWMLEDS) { // Don't start if in progress
		log_byte(0x80 + current_slow_adc);
	} else {
		current_slow_adc = NUM_ADCS;
		// TODO: kick the watchdog here
	}
}
#endif

/*
 * Single synchronous ADC conversion.
 * Has to be called with IRQs disabled (or with the ADC IRQ disabled).
 */
static uint16_t read_adc_sync()
{
	uint16_t rv;

	ADCSRA |= _BV(ADSC); // start the conversion

	// wait for the conversion to finish
	while((ADCSRA & _BV(ADIF)) == 0)
		;

	rv = ADCW;
	ADCSRA |= _BV(ADIF); // clear the IRQ flag

	return rv;
}

void init_adc()
{
	current_slow_adc = NUM_ADCS;
	current_adc = 0;

	ADCSRA = _BV(ADEN)			// enable
		| _BV(ADPS1) | _BV(ADPS0)	// CLK/8 = 125 kHz
		// | _BV(ADPS2)			// CLK/16 = 62.5 kHz
		;
	// ADCSRB |= _BV(GSEL); // gain 8 or 32

	// Disable digital input on all bits used by ADC
	DIDR0 = _BV(ADC3D) | _BV(ADC2D);
	
	// 1.1V, GND
	ADMUX = _BV(REFS1) | _BV(MUX3) | _BV(MUX2) | _BV(MUX0);

	/* Do first conversion and drop the result */
	read_adc_sync();

	ADCSRA |= _BV(ADIE); // enable IRQ

	start_next_adc();
}

#if 0
void susp_adc()
{
	ADCSRA = 0;
	DIDR0 = 0;
}

static void adc1_gain20_adc(uint16_t adcsum)
{
	// running average
	adc1_gain20_offset += adcsum
			- (adc1_gain20_offset >> ADC1_GAIN20_OFFSET_SHIFT);
}
#endif

static void inline adc_based_timer()
{
	static uint16_t pattern_counter;

	if (++pattern_counter > 250) {
		pattern_counter = 0;
		patterns_next_tick();
	}
}

ISR(ADC_vect) { // IRQ handler
	uint16_t adcval = ADCW;

	adc_based_timer();

	if (read_zero) {
		setup_mux(current_adc);
		read_zero = 0;
		ADCSRA |= _BV(ADSC); // drop this one, start the next
		return;
	}

	if (drop_count) {
		ADCSRA |= _BV(ADSC); // drop this one, start the next
		drop_count--;
		return;
	}

	if (read_count) {
		ADCSRA |= _BV(ADSC); // immediately start the next conversion
		adc_sum += adcval;
		read_count--;
		return;
	}

	/*
	 * Now we have performed read_count measurements and have them
	 * in adc_sum.
	 */
	switch (current_adc) {
	case 0:
		// pwmled_adc(current_adc, adc_sum);
		pwmled_adc(adc_sum);
		break;
	}

	start_next_adc();
}
